from typing import Optional, Iterable

from .client import TraYan
from trayan.models.translator import Language


def detect(
        text: str,
        hints: Optional[Iterable[str]] = None,
        proxy: Optional[Iterable[str]] = None,
) -> 'Language':
    with TraYan(proxy=proxy) as t:
        return t.detect(text=text, hints=hints)


def translate(
        text: str,
        source_language: str = Language.EN,
        target_language: str = Language.RU,
        proxy: Optional[Iterable[str]] = None,
) -> str:
    with TraYan(proxy=proxy) as t:
        return t.translate(text=text, source_language=source_language, target_language=target_language)



import os
from typing import Optional, Iterable

from httpx import Client

from trayan.models.errors import SidParseError, BadResponse
from trayan.models.translator import Language
from trayan.utils import cached_property
from ..base import BaseTraYan


class TraYan(BaseTraYan):
    def detect(self, text: str, hints: Optional[Iterable[str]] = None) -> 'Language':
        req = self._detect_request
        req.url = req.url.copy_merge_params(
            params=dict(
                sid=self.sid,
                text=text,
            ) | (dict(hint=hints if isinstance(hints, str) else ','.join(hints)) if hints else {})
        )
        resp = self._session.send(req)
        if resp.status_code == 200:
            return resp.json().get('lang')
        raise BadResponse(resp.content.decode('utf-8'))

    def translate(
            self,
            text: str,
            source_language: str = Language.EN,
            target_language: str = Language.RU
    ) -> str:
        req = self._translate_request
        req.url = req.url.copy_merge_params(
            params=dict(
                id=self.id,
                lang=f'{source_language}-{target_language}',
                text=text,
            )
        )
        resp = self._session.send(req)
        if resp.status_code == 200:
            return resp.json().get('text')[0]
        raise BadResponse(resp.content.decode('utf-8'))

    def __enter__(self):
        self.close()
        self.start()
        return self

    def __exit__(self, *err):
        self.close()

    def _init_session(self, is_ssl: bool = True) -> Client:
        return Client(proxies=self._proxy)

    def close(self) -> None:
        if self._session:
            self._session.close()
        self._session = None

    @cached_property(ttl=60 * 60 * 24 * 4)
    def sid(self) -> str:
        if not self._check_cache():
            sid = self._gen_sid()
            try:
                self._save_cache(sid)
            except PermissionError:
                ...
            return sid
        return self._get_cache()

    @cached_property
    def id(self) -> str:
        return f'{self.sid}{self._suffix}'

    def _save_cache(self, data: str) -> None:
        with open(self._CACHE_PATH, 'w', encoding='utf8') as f:
            os.utime(self._CACHE_PATH, None)
            f.write(data)

    def _get_cache(self) -> Optional[str]:
        with open(self._CACHE_PATH, 'r', encoding='utf8') as f:
            return f.read()

    def _gen_sid(self) -> str:
        resp = self._session.send(self._get_sid_request)
        if sid := self._sid_reg_exp.findall(resp.content.decode('utf-8')):
            return '.'.join(i[::-1] for i in sid[0].split('.'))
        raise SidParseError()



from asyncio import iscoroutinefunction, ensure_future, Future
from time import time
from typing import Optional, Callable, Union, Awaitable, Any


class cached_property:
    def __init__(self, ttl: Optional[int] = None):
        if callable(ttl):
            func = ttl
            ttl = None
        else:
            func = None
        self.ttl = ttl
        self._prepare_func(func)

    def __call__(self, func, doc=None):
        self._prepare_func(func, doc)
        return self

    def __get__(self, obj, cls):
        now = time()
        if cache := obj.__dict__.get(self.__name__):
            cache_value, last_updated = cache
            if not (self.ttl and self.ttl < now - last_updated):
                return cache_value
        cache = ensure_future(self.func(obj)) if iscoroutinefunction(self.func) else self.func(obj)
        self._update_cache(obj, cache, now)
        return cache

    def __set__(self, obj, value):
        now = time()
        if iscoroutinefunction(self.func):
            future = Future()
            future.set_result(value)
            value = future
        self._update_cache(obj, value, now)

    def __delete__(self, obj):
        obj.__dict__.pop(self.__name__, None)

    def _prepare_func(self, func: Union[Callable, Awaitable], doc: Optional[str] = None) -> None:
        self.func = func
        if func:
            self.__doc__ = doc or getattr(func, '__doc__')
            self.__name__ = func.__name__
            self.__module__ = func.__module__

    def _update_cache(self, obj: Callable, cache: Any, now: float) -> None:
        obj.__dict__[self.__name__] = (cache, now)



import os
import re
from abc import ABCMeta, abstractmethod
from base64 import b64decode
from random import randint
from time import time
from typing import Optional, Dict

from httpx import Request
from httpx._types import ProxiesTypes

from trayan.models.request import RequestMethod
from trayan.models.translator import ApiMethod


class BaseTraYan:
    __metaclass__ = ABCMeta

    BASE_SITE_URL = b64decode(b'aHR0cHM6Ly90cmFuc2xhdGUueWFuZGV4LnJ1Lw==').decode('utf-8')
    BASE_API_URL = b64decode(b'aHR0cHM6Ly90cmFuc2xhdGUueWFuZGV4Lm5ldC9hcGkvdjEvdHIuanNvbi8=').decode('utf-8')
    SUPPORTED_LANGS = {
        'az': 'Azerbaijani',
        'ml': 'Malayalam',
        'sq': 'Albanian',
        'mt': 'Maltese',
        'am': 'Amharic',
        'mk': 'Macedonian',
        'en': 'English',
        'mi': 'Maori',
        'ar': 'Arabic',
        'mr': 'Marathi',
        'hy': 'Armenian',
        'mhr': 'Mari',
        'af': 'Afrikaans',
        'mn': 'Mongolian',
        'eu': 'Basque',
        'de': 'German',
        'ba': 'Bashkir',
        'ne': 'Nepalese',
        'be': 'Belarusian',
        'no': 'Norwegian',
        'bn': 'Bengal',
        'pa': 'Punjabi',
        'my': 'Burmese',
        'pap': 'Papiamento',
        'bg': 'Bulgarian',
        'fa': 'Persian',
        'bs': 'Bosnian',
        'pl': 'Polish',
        'cy': 'Welsh',
        'pt': 'Portuguese',
        'hu': 'Hungarian',
        'ro': 'Romanian',
        'vi': 'Vietnamese',
        'ru': 'Russian',
        'ht': 'Haitian (Creole)',
        'ceb': 'Cebuano',
        'gl': 'Galician',
        'sr': 'Serbian',
        'nl': 'Dutch',
        'si': 'Sinhalese',
        'mrj': 'Hill Mari',
        'sk': 'Slovak',
        'el': 'Greek',
        'sl': 'Slovenian',
        'ka': 'Georgian',
        'sw': 'Swahili',
        'gu': 'Gujarati',
        'su': 'Sundanese',
        'da': 'Danish',
        'tg': 'Tajik',
        'he': 'Hebrew',
        'th': 'Thai',
        'yi': 'Yiddish',
        'tl': 'Tagalog',
        'id': 'Indonesian',
        'ta': 'Tamil',
        'ga': 'Irish',
        'tt': 'Tartar',
        'it': 'Italian',
        'te': 'Telugu',
        'is': 'Icelandic',
        'tr': 'Turkish',
        'es': 'Spanish',
        'udm': 'Udmurt',
        'kk': 'Kazakh',
        'uz': 'Uzbek',
        'kn': 'Kannada',
        'uk': 'Ukrainian',
        'ca': 'Catalan',
        'ur': 'Urdu',
        'ky': 'Kirghiz',
        'fi': 'Finnish',
        'zh': 'Chinese',
        'fr': 'French',
        'ko': 'Korean',
        'hi': 'Hindi',
        'xh': 'Xhosa',
        'hr': 'Croatian',
        'km': 'Khmer',
        'cs': 'Czech',
        'lo': 'Laotian',
        'sv': 'Swedish',
        'la': 'Latin',
        'gd': 'Scottish',
        'lv': 'Latvian',
        'et': 'Estonian',
        'lt': 'Lithuanian',
        'eo': 'Esperanto',
        'lb': 'Luxembourg',
        'jv': 'Javanese',
        'mg': 'Malagasy',
        'ja': 'Japanese',
        'ms': 'Malay'
    }

    USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0'
    _HEADERS = {
        'Host': 'translate.yandex.ru',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,*/*;q=0.8',
        'Accept-Language': 'en-US,en;q=0.5',
        'Accept-Encoding': 'gzip, deflate, br',
        'User-Agent': USER_AGENT,
    }
    _API_HEADERS = {
        'Host': 'translate.yandex.net',
        'Accept': '*/*',
        'Accept-Language': 'en-US,en;q=0.5',
        'Accept-Encoding': 'gzip, deflate, br',
        'Origin': 'https://translate.yandex.ru',
        'Referer': 'https://translate.yandex.ru/',
        'User-Agent': USER_AGENT,
    }

    _SID_REG_EXP = r'SID\s?:\s?[\'"]([^\'"]+)[\'"]'

    SID_EXPIRES_TIME = 60 * 60 * 24 * 4

    _CACHE_PATH = os.path.join(os.path.expanduser('~'), '.trayan-old.key')

    def __init__(self, proxy: Optional[ProxiesTypes] = None):
        self._session = None
        self._proxy = proxy

        self._sid_reg_exp = re.compile(self._SID_REG_EXP)

        self._get_sid_request = Request(
            RequestMethod.GET,
            self.BASE_SITE_URL,
            headers=self._HEADERS,
        )
        self._detect_request = Request(
            RequestMethod.GET,
            f'{self.BASE_API_URL}{ApiMethod.DETECT}',
            headers=self._API_HEADERS,
            params=dict(
                srv='tr-text',
                options=1,
            ),
        )
        self._translate_request = Request(
            RequestMethod.POST,
            f'{self.BASE_API_URL}{ApiMethod.TRANSLATE}',
            headers=self._API_HEADERS,
            params=dict(
                srv='tr-text',
                reason='type-end',
                format='text',
                ajax=1,
                options=4,
            ),
        )

        self.start()

    def start(self) -> None:
        self._session = self._init_session()

    @abstractmethod
    def _init_session(self):
        raise NotImplementedError

    @abstractmethod
    def close(self):
        raise NotImplementedError

    @abstractmethod
    def detect(self, *args, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def translate(self, *args, **kwargs):
        raise NotImplementedError

    @property
    def _suffix(self) -> str:
        return f'-{randint(0, 9)}-0'

    @classmethod
    @property
    def supported_langs(cls) -> Dict[str, str]:
        return cls.SUPPORTED_LANGS

    @classmethod
    def get_supported_langs(cls) -> Dict[str, str]:
        return cls.supported_langs

    def _check_cache(self) -> bool:
        if os.path.exists(self._CACHE_PATH):
            try:
                return os.path.isfile(self._CACHE_PATH) & \
                    ((time() - os.path.getmtime(self._CACHE_PATH)) < self.SID_EXPIRES_TIME)
            except PermissionError:
                ...



import sys

from setuptools import setup, find_packages

if sys.version_info[0] != 3:
    print('This script requires Python >= 3')
    exit(1)

with open('README.md', 'r') as readme_file:
    readme = readme_file.read()

requirements = ['httpx[socks]>=0.23.3', 'aiofiles>=22.0.1']

setup(
    name='tra-yan',
    version='0.1.0',
    author='re1von project',
    author_email='re1von_project@bk.ru',
    url='https://github.com/re1von/tra-yan',
    long_description=readme,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
)
